@@ -206,12 +206,14 @@ gnomemeeting_warning_dialog_on_widget (GtkWindow *parent,
     g_strdup_printf ("%s\n\n%s", prim_text, buffer);
 
   dialog = gtk_message_dialog_new (parent, 
-                                   GTK_DIALOG_MODAL,
+                                   0,
                                    GTK_MESSAGE_WARNING,
                                    GTK_BUTTONS_OK,
                                    NULL);
-  gtk_window_set_title (GTK_WINDOW (dialog), "");
+  gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+  gtk_window_present (GTK_WINDOW (dialog));
   
+  gtk_window_set_title (GTK_WINDOW (dialog), "");
   gtk_label_set_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label),
 			dialog_text);
 
@@ -267,6 +269,7 @@ gnomemeeting_dialog (GtkWindow *parent,
   dialog =
     gtk_message_dialog_new (parent, GTK_DIALOG_MODAL, type,
 			    GTK_BUTTONS_OK, NULL);
+
   gtk_window_set_title (GTK_WINDOW (dialog), "");
   gtk_label_set_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label),
 			dialog_text);
