@@ -1,6 +1,6 @@
 /*
  * XML LDAP client
- * $Id: xdap.c,v 1.9.2.2 2003/09/09 09:19:37 dsandras Exp $
+ * $Id: xdap.c,v 1.12 2003/11/18 22:12:02 dsandras Exp $
  * Copyright 2001 paul666@mailandnews.com
  * Licensed under version 2 or later of the GNU GPL
  */
@@ -156,7 +156,7 @@ parseonly (char *filename, xmlEntityPtr (*getent) (void *, const xmlChar *),
 	   xmlEntityPtr (**oldgetent) (void *, const xmlChar *), int noerr)
 {
   xmlDocPtr xp;
-  xmlSAXHandler sax;
+  xmlSAXHandlerV1 sax;
 
   memset (&sax, 0, sizeof sax);
   sax = xmlDefaultSAXHandler;	/* Copy original handler */
