@@ -44,16 +44,16 @@
 #include "gnomemeeting.h"
 #include "misc.h"
 #include "menu.h"
-#include "gtk-text-buffer-extentions.h"
 
+#include "gtk-text-buffer-extentions.h"
+#include "gtk_menu_extensions.h"
 
 extern GtkWidget *gm;
-extern GnomeMeeting *MyApp;
 
 
 static void chat_entry_activate (GtkEditable *w, gpointer data)
 {
-  GMH323EndPoint *endpoint = MyApp->Endpoint ();
+  GMH323EndPoint *endpoint = GnomeMeeting::Process ()->Endpoint ();
   PString s;
     
   if (endpoint) {
@@ -65,10 +65,16 @@ static void chat_entry_activate (GtkEditable *w, gpointer data)
 
     s = PString (gtk_entry_get_text (GTK_ENTRY (w)));
     
-    if (endpoint->GetCallingState () == 2 && !s.IsEmpty ()) {
-            
-      H323Connection *connection = endpoint->GetCurrentConnection ();
+    if (endpoint->GetCallingState () == GMH323EndPoint::Connected
+	&& !s.IsEmpty ()) {
             
+      /* If the GDK lock is taken, the connection will never get a
+	 chance to be established if we lock its mutex */
+      gdk_threads_leave ();
+      H323Connection *connection = 
+	endpoint->FindConnectionWithLock (endpoint->GetCurrentCallToken ());
+      gdk_threads_enter ();
+
       if (connection != NULL)  {
                 
 	connection->SendUserInput ("MSG"+s);
@@ -83,6 +89,8 @@ static void chat_entry_activate (GtkEditable *w, gpointer data)
 	g_free (utf8_local);
                 
 	gtk_entry_set_text (GTK_ENTRY (w), "");
+
+	connection->Unlock ();
       }
     }
   }
@@ -91,7 +99,10 @@ static void chat_entry_activate (GtkEditable *w, gpointer data)
 
 void gnomemeeting_text_chat_clear (GtkWidget *w, GmTextChat *chat)
 {
+  GmWindow *gw = NULL;
   GtkTextIter start_iter, end_iter;
+
+  gw = GnomeMeeting::Process ()->GetMainWindow ();
   
   gtk_text_buffer_get_start_iter (chat->text_buffer, &start_iter);
   gtk_text_buffer_get_end_iter (chat->text_buffer, &end_iter);
@@ -99,7 +110,7 @@ void gnomemeeting_text_chat_clear (GtkWidget *w, GmTextChat *chat)
   gtk_text_buffer_delete (chat->text_buffer, &start_iter, &end_iter);
   chat->buffer_is_empty = TRUE;
 
-  gnomemeeting_view_menu_text_chat_clear_set_sensitive (FALSE);
+  gtk_menu_set_sensitive (gw->main_menu, "clear_text_chat", FALSE);
 }
 
 
@@ -110,7 +121,11 @@ gnomemeeting_text_chat_insert (PString local, PString str, int user)
   GtkTextIter iter;
   GtkTextMark *mark;
   
-  GmTextChat *chat = MyApp->GetTextChat ();
+  GmTextChat *chat = NULL;
+  GmWindow *gw = NULL;
+
+  gw = GnomeMeeting::Process ()->GetMainWindow ();
+  chat = GnomeMeeting::Process ()->GetTextChat ();
 
   gtk_text_buffer_get_end_iter (chat->text_buffer, &iter);
 
@@ -131,7 +146,7 @@ gnomemeeting_text_chat_insert (PString local, PString str, int user)
   
   g_free (msg);
   
-  gtk_text_buffer_insert_with_emoticons (chat->text_buffer, &iter, 
+  gtk_text_buffer_insert_with_addons (chat->text_buffer, &iter, 
 					 (const char *) str);
 
   mark = gtk_text_buffer_get_mark (chat->text_buffer, "current-position");
@@ -139,7 +154,7 @@ gnomemeeting_text_chat_insert (PString local, PString str, int user)
   gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (chat->text_view), mark, 
 				0.0, FALSE, 0,0);
 
-  gnomemeeting_view_menu_text_chat_clear_set_sensitive (TRUE);
+  gtk_menu_set_sensitive (gw->main_menu, "clear_text_chat", TRUE);
 }
 
 
@@ -188,7 +203,7 @@ GtkWidget *gnomemeeting_text_chat_new (GmTextChat *chat)
 			      "weight", 900, NULL);
 
   gtk_text_buffer_create_tag (chat->text_buffer, "secondary-user",
-			      "foreground", "blue", 
+			      "foreground", "#38b749", 
 			      "weight", 900, NULL);
 
   frame = gtk_frame_new (NULL);
