@@ -1,4 +1,3 @@
-
 /* GnomeMeeting -- A Video-Conferencing application
  * Copyright (C) 2000-2003 Damien Sandras
  *
@@ -52,10 +51,12 @@
 #include "main_window.h"
 #include "toolbar.h"
 #include "misc.h"
+
 #include "history-combo.h"
 #include "dialog.h"
 #include "e-splash.h"
 #include "stock-icons.h"
+#include "gconf_widgets_extensions.h"
 
 #ifndef WIN32
 #include <esd.h>
@@ -66,9 +67,8 @@
 static gint
 gnomemeeting_tray_hack (gpointer);
 
+GnomeMeeting *GnomeMeeting::GM = NULL;
 
-/* Declarations */
-GnomeMeeting *MyApp;	
 GtkWidget *gm;
 
 
@@ -79,12 +79,12 @@ gnomemeeting_tray_hack (gpointer data)
 
   gdk_threads_enter ();
 
-  gw = MyApp->GetMainWindow ();
+  gw = GnomeMeeting::Process ()->GetMainWindow ();
   
   if (!gnomemeeting_tray_is_visible (gw->docklet)) {
 
     gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Notification area not detected"), _("You have chosen to start GnomeMeeting hidden, however the notification area is not present in your panel, GnomeMeeting can thus not start hidden."));
-    gtk_widget_show (gm);
+    gnomemeeting_window_show (gm);
   }
   
   gdk_threads_leave ();
@@ -116,28 +116,28 @@ GnomeMeeting::GnomeMeeting ()
   memset ((void *) rtp, 0, sizeof (struct _GmRtpData));
   gw->progress_timeout = 0;
   gw->docklet = gw->ldap_window = gw->pref_window = gw->calls_history_window =
-    gw->splash_win = gw->incoming_call_popup = gw->history_window =
+    gw->splash_win = gw->incoming_call_popup = gw->history_window = 
 #ifndef DISABLE_GNOME
     gw->druid_window =
 #endif
     NULL;
-
+							     
   call_number = 0;
 
-  MyApp = (this);
+  GM = this;
 }
 
 
 GnomeMeeting::~GnomeMeeting()
 {
   if (endpoint) {
-    
-    endpoint->ClearAllCalls (H323Connection::EndedByLocalUser, TRUE);
-    endpoint->RemoveVideoGrabber (true);
+
+    endpoint->RemoveVideoGrabber ();
 #ifdef HAS_IXJ
     endpoint->RemoveLid ();
 #endif
   }
+  
   RemoveEndpoint ();
 
   if (gw->ldap_window) {
@@ -172,12 +172,6 @@ void
 GnomeMeeting::Connect()
 {
   PString call_address;
-  PString current_call_token;
-  H323Connection *connection = NULL;
-    
-  /* We need a connection to use AnsweringCall */
-  current_call_token = endpoint->GetCurrentCallToken ();
-  connection = endpoint->GetCurrentConnection ();
   
   gnomemeeting_threads_enter ();
   gnomemeeting_statusbar_push  (gw->statusbar, NULL);
@@ -185,8 +179,8 @@ GnomeMeeting::Connect()
   gnomemeeting_threads_leave ();
 
 
-  /* If connection, then answer it */
-  if (connection != NULL) {
+  /* If incoming connection, then answer it */
+ if (endpoint->GetCallingState () == 3) {
 
     gnomemeeting_threads_enter ();
     gnomemeeting_log_insert (gw->history_text_view,
@@ -242,91 +236,115 @@ GnomeMeeting::Connect()
 void
 GnomeMeeting::Disconnect (H323Connection::CallEndReason reason)
 {
-  /* If somebody is calling us, then we do not accept the connection
-     else we finish it */
-  H323Connection *connection = endpoint->GetCurrentConnection ();
-  PString current_call_token = endpoint->GetCurrentCallToken ();
-
-
   gnomemeeting_threads_enter ();
   gnomemeeting_main_window_enable_statusbar_progress (false);
   gnomemeeting_statusbar_push (gw->statusbar, NULL);
   gnomemeeting_threads_leave ();
 
 
-  if (!current_call_token.IsEmpty ()) {
+  /* if we are trying to call somebody */
+  if (endpoint->GetCallingState () == 1) {
 
-    /* if we are trying to call somebody */
-    if (endpoint->GetCallingState () == 1) {
+    gnomemeeting_threads_enter ();
+    gnomemeeting_log_insert (gw->history_text_view,
+			     _("Trying to stop calling"));
+    gnomemeeting_threads_leave ();
+
+    endpoint->ClearCall (endpoint->GetCurrentCallToken (), reason);
+  }
+  else {
+
+    /* if we are in call with somebody */
+    if (endpoint->GetCallingState () == 2) {
+
+      gnomemeeting_threads_enter ();	
+      gnomemeeting_log_insert (gw->history_text_view,
+			       _("Stopping current call"));
+      connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 
+				    0);
+      gnomemeeting_threads_leave ();
+
+      endpoint->ClearAllCalls (reason, FALSE);
+    }
+    else if (endpoint->GetCallingState () == 3) {
 
       gnomemeeting_threads_enter ();
       gnomemeeting_log_insert (gw->history_text_view,
-			       _("Trying to stop calling"));
+			       _("Refusing Incoming call"));
+      connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 
+				    0);
       gnomemeeting_threads_leave ();
 
-      endpoint->ClearCall (current_call_token, reason);
+      endpoint->ClearCall (endpoint->GetCurrentCallToken (),
+			   H323Connection::EndedByLocalUser);
     }
-    else {
+  }
+}
 
-      /* if somebody is calling us, or if we are in call with somebody */
-      if (endpoint->GetCallingState () == 2
-	  || endpoint->GetCallingState () == 3) {
 
-	gnomemeeting_threads_enter ();	
-	gnomemeeting_log_insert (gw->history_text_view,
-				 _("Stopping current call"));
-	connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 
-				      0);
-	gnomemeeting_threads_leave ();
+void
+GnomeMeeting::Init ()
+{
+#ifndef WIN32
+  /* Ignore SIGPIPE */
+  signal (SIGPIPE, SIG_IGN);
+#endif
 
-	/* End of Call */
-	endpoint->ClearAllCalls (reason, FALSE);
-      }
-      else {
-
-	gnomemeeting_threads_enter ();
-	gnomemeeting_log_insert (gw->history_text_view,
-				 _("Refusing Incoming call"));
-	connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 
-				      0);
-	gnomemeeting_threads_leave ();
-
-	/* Either the user clicks on disconnect when we are called,
-	   either the reason is different */
-	if (reason == H323Connection::EndedByLocalUser && connection)
-	  connection->AnsweringCall (H323Connection::AnswerCallDenied);
-	else
-	  endpoint->ClearAllCalls (reason, FALSE);
-      }
-    }
-  } 
+  endpoint = new GMH323EndPoint ();
+  endpoint->Init ();
 }
 
 
 BOOL
 GnomeMeeting::DetectDevices ()
 {
+  gchar *audio_plugin = NULL;
+  gchar *video_plugin = NULL;
+
+  PINDEX fake_idx;
+
+  audio_plugin = gconf_get_string (AUDIO_DEVICES_KEY "plugin");
+  video_plugin = gconf_get_string (VIDEO_DEVICES_KEY "plugin");
+ 
+  if (!audio_plugin || !video_plugin)
+    return FALSE;
+
   /* Detect the devices */
   gnomemeeting_sound_daemons_suspend ();
-  gw->audio_player_devices = gnomemeeting_get_audio_player_devices ();
-  gw->audio_recorder_devices = gnomemeeting_get_audio_recorder_devices ();
-  gw->video_devices = PVideoInputDevice::GetInputDeviceNames ();
-#ifdef TRY_1394DC
-  gw->video_devices += PVideoInput1394DcDevice::GetInputDeviceNames();
-#endif
-#ifdef TRY_1394AVC
-  gw->video_devices += PVideoInput1394AvcDevice::GetInputDeviceNames();
-#endif
-  gw->video_devices += PString (_("Picture"));
-  gw->audio_mixers = gnomemeeting_get_mixers ();
-  gnomemeeting_mixers_mic_select ();
+
+  /* Detect the plugins */
+  gw->audio_managers = PSoundChannel::GetDriverNames ();
+  gw->video_managers = PVideoInputDevice::GetDriverNames ();
+
+  /* Detect the devices */
+  gw->video_devices = PVideoInputDevice::GetDeviceNames (video_plugin);
+
+  gw->audio_recorder_devices = 
+    PSoundChannel::GetDeviceNames (audio_plugin, PSoundChannel::Recorder);
+  gw->audio_player_devices = 
+    PSoundChannel::GetDeviceNames (audio_plugin, PSoundChannel::Player);
+
+  if (gw->audio_recorder_devices.GetSize () == 0) 
+    gw->audio_recorder_devices += PString (_("No device found"));
+  if (gw->audio_player_devices.GetSize () == 0)
+    gw->audio_player_devices += PString (_("No device found"));
+  if (gw->video_devices.GetSize () == 0)
+    gw->video_devices += PString (_("No device found"));
+
+  g_free (audio_plugin);
+  g_free (video_plugin);
+
+  if (gw->audio_managers.GetSize () == 0)
+    return FALSE;
+
+  fake_idx = gw->video_managers.GetValuesIndex (PString ("FakeVideo"));
+  if (fake_idx != P_MAX_INDEX)
+    gw->video_managers.RemoveAt (fake_idx);
+  
+  gw->audio_managers += PString ("Quicknet");
   gnomemeeting_sound_daemons_resume ();
 
-  if (gw->audio_player_devices.GetSize () == 0
-      || gw->audio_recorder_devices.GetSize () ==0)
-    return FALSE;
-  else
-    return TRUE;
+  return TRUE;
 }
 
 
@@ -342,6 +360,13 @@ GnomeMeeting::Endpoint ()
 }
 
 
+GnomeMeeting *
+GnomeMeeting::Process ()
+{
+  return GM;
+}
+
+
 GmWindow *
 GnomeMeeting::GetMainWindow ()
 {
@@ -391,41 +416,8 @@ GnomeMeeting::GetRtpData ()
 }
 
 
-void GnomeMeeting::Main () {}
-
-
-void
-GnomeMeeting::InitComponents ()
+void GnomeMeeting::Main ()
 {
-#ifndef WIN32
-  /* Ignore SIGPIPE */
-  signal (SIGPIPE, SIG_IGN);
-#endif
-  
-
-  endpoint = new GMH323EndPoint ();
-
-  /* Start the video preview */
-  if (gconf_client_get_bool (client, DEVICES_KEY "video_preview", NULL))
-    endpoint->CreateVideoGrabber ();
-
-  endpoint->SetUserNameAndAlias ();
-
-  /* Register to gatekeeper */
-  if (gconf_client_get_int (client, GATEKEEPER_KEY "registering_method", 0))
-    endpoint->GatekeeperRegister ();
-
-  /* The LDAP part, if needed */
-  if (gconf_client_get_bool (GCONF_CLIENT (client), LDAP_KEY "register", NULL)) 
-  {
-      GMILSClient *gm_ils_client = 
-	GM_ILS_CLIENT (endpoint->GetILSClientThread ());
-      gm_ils_client->Register ();
-  }
-  
-  
-  if (!endpoint->StartListener ()) 
-    gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Error while starting the listener"), _("You will not be able to receive incoming calls. Please check that no other program is already running on the port used by GnomeMeeting."));
 }
 
 
@@ -458,11 +450,11 @@ void GnomeMeeting::BuildGUI ()
   gw->calls_history_window = gnomemeeting_calls_history_window_new (chw);  
   gw->pref_window = gnomemeeting_pref_window_new (pw);  
   gw->ldap_window = gnomemeeting_ldap_window_new (lw);
+#ifndef DISABLE_GNOME
   gw->druid_window = gnomemeeting_druid_window_new (dw);
+#endif
 #ifndef WIN32
   gw->docklet = gnomemeeting_init_tray ();
-  if (gconf_client_get_bool (client, GENERAL_KEY "do_not_disturb", 0))
-    gnomemeeting_tray_set_content (gw->docklet, 2);
 #endif
   gnomemeeting_main_window_new (gw);
 
@@ -477,12 +469,10 @@ void GnomeMeeting::BuildGUI ()
 #endif
     /* Show the main window */
 #ifndef WIN32
-    if (!gconf_client_get_bool (GCONF_CLIENT (client),
-				VIEW_KEY "start_docked", 0)) {
+    if (!gconf_get_bool (USER_INTERFACE_KEY "start_hidden")) 
 #endif
-      gtk_widget_show (GTK_WIDGET (gm));
+      gnomemeeting_window_show (gm);
 #ifndef WIN32
-    }
     else
       gtk_timeout_add (15000, (GtkFunction) gnomemeeting_tray_hack, NULL);
 #endif
