@@ -45,16 +45,16 @@
 #include "sound_handling.h"
 #include "main_window.h"
 #include "callbacks.h"
-#include "dialog.h"
 
-#define new PNEW
+#include "dialog.h"
+#include "gconf_widgets_extensions.h"
+
 
 static gboolean transfer_callback_cb (gpointer data);
 
 
 /* Declarations */
 extern GtkWidget *gm;
-extern GnomeMeeting *MyApp;
 
 
 /* DESCRIPTION  :  /
@@ -108,18 +108,15 @@ void GMLid::Open ()
   
   gnomemeeting_threads_enter ();
   client = gconf_client_get_default ();
-  gw = MyApp->GetMainWindow ();
+  gw = GnomeMeeting::Process ()->GetMainWindow ();
   gnomemeeting_threads_leave ();
 
   if (!lid) {
 
     gnomemeeting_threads_enter ();
-    lid_device =  
-      gconf_client_get_string (client, DEVICES_KEY "audio_player", NULL);
-    lid_country =
-      gconf_client_get_string (client, DEVICES_KEY "lid_country", NULL);
-    lid_aec =
-      gconf_client_get_int (client, DEVICES_KEY "lid_aec", NULL);
+    lid_device = gconf_get_string (AUDIO_DEVICES_KEY "input_device");
+    lid_country = gconf_get_string (AUDIO_DEVICES_KEY "lid_country_code");
+    lid_aec = gconf_get_int (AUDIO_DEVICES_KEY "lid_echo_cancellation_level");
     gnomemeeting_threads_leave ();
 
     if (lid_device == NULL)
@@ -186,36 +183,10 @@ void GMLid::Stop ()
 
 void GMLid::Close ()
 {
-  GConfClient *client = NULL;
-  GmWindow *gw = NULL;
-  gchar *mixer = NULL;
-  int vol = 0;
-
   PWaitAndSignal m(device_access_mutex);
   
   if (lid)
     lid->Close ();
-
-  /* Restore the normal mixers settings */
-  gnomemeeting_threads_enter ();
-  client = gconf_client_get_default ();
-  gw = MyApp->GetMainWindow ();
-
-  mixer =
-    gconf_client_get_string (client, DEVICES_KEY "audio_player_mixer", NULL);
-  vol = gnomemeeting_get_mixer_volume (mixer, SOURCE_AUDIO);
-  g_free (mixer);
-
-  GTK_ADJUSTMENT (gw->adj_play)->value = (int) (vol & 255);
-
-  mixer =
-    gconf_client_get_string (client, DEVICES_KEY "audio_recorder_mixer", NULL);
-  vol = gnomemeeting_get_mixer_volume (mixer, SOURCE_MIC);
-  g_free (mixer);
-
-  GTK_ADJUSTMENT (gw->adj_rec)->value = (int) (vol & 255);
-  gtk_widget_queue_draw (GTK_WIDGET (gw->audio_settings_frame));
-  gnomemeeting_threads_leave (); 
 }
 
 
@@ -241,7 +212,7 @@ void GMLid::Main ()
   OffHook = lastOffHook = lid->IsLineOffHook (OpalIxJDevice::POTSLine);
 
   gnomemeeting_threads_enter ();
-  gw = MyApp->GetMainWindow ();
+  gw = GnomeMeeting::Process ()->GetMainWindow ();
 
   /* Update the mixers if the lid is used */
   lid->GetPlayVolume (0, vol);
@@ -254,7 +225,7 @@ void GMLid::Main ()
 
   while (lid && lid->IsOpen() && !stop) {
 
-    endpoint = MyApp->Endpoint ();
+    endpoint = GnomeMeeting::Process ()->Endpoint ();
     calling_state = endpoint->GetCallingState ();
 
     OffHook = (lid->IsLineOffHook (0));
@@ -283,7 +254,7 @@ void GMLid::Main ()
       if (calling_state == 3) { /* 3 = incoming call */
 
 	lid->StopTone (0);
-	MyApp->Connect ();
+	GnomeMeeting::Process ()->Connect ();
       }
 
 
@@ -297,7 +268,7 @@ void GMLid::Main ()
 	gnomemeeting_threads_leave ();
 	
 	if (url && !GMURL (url).IsEmpty ())
-	  MyApp->Connect ();
+	  GnomeMeeting::Process ()->Connect ();
       }
     }
 
@@ -318,7 +289,7 @@ void GMLid::Main ()
 
       if (calling_state == 2 || calling_state == 1) {
 
-	MyApp->Disconnect ();
+	GnomeMeeting::Process ()->Disconnect ();
       }
     }
 
@@ -330,7 +301,7 @@ void GMLid::Main ()
       if ((t.GetSeconds () > 5 && !do_not_connect) || c == '#') {
 
 	if (calling_state == 0)
-	  MyApp->Connect ();
+	  GnomeMeeting::Process ()->Connect ();
 	do_not_connect = TRUE;
       }
 
