@@ -49,9 +49,11 @@
 #include "menu.h"
 #include "callbacks.h"
 #include "sound_handling.h"
+
 #include "tray.h"
 #include "dialog.h"
 #include "stock-icons.h"
+#include "gconf_widgets_extensions.h"
 
 
 #ifndef DISABLE_GNOME
@@ -77,9 +79,9 @@
 
 /* Declarations */
 extern GtkWidget *gm;
-extern GnomeMeeting *MyApp;	
 
 
+static void video_window_shown_cb (GtkWidget *, gpointer);
 static gboolean stats_drawing_area_exposed (GtkWidget *, gpointer data);
 
 #ifndef DISABLE_GNOME
@@ -99,13 +101,11 @@ static void colour_changed             (GtkAdjustment *, gpointer);
 static void contrast_changed           (GtkAdjustment *, gpointer);
 static void dialpad_button_clicked     (GtkButton *, gpointer);
 
-static int
-gnomemeeting_main_window_appbar_update (gpointer); 
 static gint gm_quit_callback (GtkWidget *, GdkEvent *, gpointer);
 static void gnomemeeting_init_main_window_video_settings ();
 static void gnomemeeting_init_main_window_audio_settings ();
 static void gnomemeeting_init_main_window_stats ();
-static void gnomemeeting_init_main_window_dialpad ();
+static void gnomemeeting_init_main_window_dialpad (GtkAccelGroup *);
 
 
 /* For stress testing */
@@ -118,7 +118,7 @@ gint StressTest (gpointer data)
    gdk_threads_enter ();
 
 
-   GmWindow *gw = MyApp->GetMainWindow ();
+   GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
 
    if (!GTK_TOGGLE_BUTTON (gw->connect_button)->active) {
 
@@ -135,174 +135,21 @@ gint StressTest (gpointer data)
 */
 
 
-gint AppbarUpdate (gpointer data)
+/* DESCRIPTION  :  This callback is called when a video window is shown.
+ * BEHAVIOR     :  Set the WM HINTS to stay-on-top if the gconf key is set
+ *                 to true.
+ * PRE          :  /
+ */
+static void
+video_window_shown_cb (GtkWidget *w, gpointer data)
 {
-  long minutes = 0, seconds = 0;
-  int jitter_buffer_size = 0;
-  RTP_Session *audio_session = NULL;
-  RTP_Session *video_session = NULL;
-  H323Connection *connection = NULL;
-  GmRtpData *rtp = (GmRtpData *) data; 
-  GmWindow *gw = NULL;
+  GMH323EndPoint *endpoint = NULL;
 
-  int received_packets = 0;
-  int lost_packets = 0;
-  int late_packets = 0;
-  float lost_packets_per = 0.0;
-  float late_packets_per = 0.0;
-  
-  if (MyApp->Endpoint ()) {
-    
-    PString current_call_token = MyApp->Endpoint ()->GetCurrentCallToken ();
+  endpoint = GnomeMeeting::Process ()->Endpoint ();
 
-    if (current_call_token.IsEmpty ())
-      return TRUE;
-    
-    gdk_threads_enter ();
-
-    gw = MyApp->GetMainWindow ();
-
-    connection = MyApp->Endpoint ()->GetCurrentConnection ();
-
-    if ((connection)&&(MyApp->Endpoint ()->GetCallingState () == 2)) {
-
-      PTimeInterval t =
-	PTime () - connection->GetConnectionStartTime();
-
-      if (t.GetSeconds () == 4) {
-
-	for (int cpt = 0 ; cpt < 50 ; cpt++) {
-
-	  rtp->tr_audio_speed [cpt] = 0;
-	  rtp->re_audio_speed [cpt] = 0;
-	  rtp->tr_video_speed [cpt] = 0;
-	  rtp->re_video_speed [cpt] = 0;
-	  rtp->tr_audio_pos = 0;
-	  rtp->re_audio_pos = 0;
-	  rtp->tr_video_pos = 0;
-	  rtp->re_video_pos = 0;
-	}
-      }
-      
-      audio_session = 
-	connection->GetSession(RTP_Session::DefaultAudioSessionID);
-	  
-      video_session = 
-	connection->GetSession(RTP_Session::DefaultVideoSessionID);
-	  
-      if (audio_session) {
-
-	/* Compute the current transmitted audio speed */
-	if ((rtp->tr_audio_bytes == 0) && (rtp->tr_audio_pos == 0))
-	  /* Default value for the 1st element */
-	  rtp->tr_audio_bytes = audio_session->GetOctetsSent();
-	rtp->tr_audio_speed [rtp->tr_audio_pos] = 
-	  (float) (audio_session->GetOctetsSent()
-		   - rtp->tr_audio_bytes)/ 1024;
-	rtp->tr_audio_bytes = 
-	  audio_session->GetOctetsSent();	
-
-	rtp->tr_audio_pos++;
-	if (rtp->tr_audio_pos >= 50) rtp->tr_audio_pos = 0;
-
-	/* Compute the current received audio speed */
-	if ((rtp->re_audio_bytes == 0) && (rtp->re_audio_pos == 0))
-	  /* Default value for the 1st element */
-	  rtp->re_audio_bytes = audio_session->GetOctetsReceived();
-	rtp->re_audio_speed [rtp->re_audio_pos] = 
-	  (float) (audio_session->GetOctetsReceived()
-		   - rtp->re_audio_bytes)/ 1024;
-	rtp->re_audio_bytes = 
-	  audio_session->GetOctetsReceived();
-
-	rtp->re_audio_pos++;
-	if (rtp->re_audio_pos >= 50) rtp->re_audio_pos = 0;
-      }
-
-      if (video_session) {
-
-	/* Compute the current transmitted video speed */
-	if ((rtp->tr_video_bytes == 0) && (rtp->tr_video_pos == 0)) 
-	  /* Default value for the 1st element */
-	  rtp->tr_video_bytes = video_session->GetOctetsSent();
-	rtp->tr_video_speed [rtp->tr_video_pos] = 
-	  (float) (video_session->GetOctetsSent()
-		   - rtp->tr_video_bytes)/ 1024;
-	rtp->tr_video_bytes = 
-	  video_session->GetOctetsSent();
-
-	rtp->tr_video_pos++;
-	if (rtp->tr_video_pos >= 50) rtp->tr_video_pos = 0;
-
-	/* Compute the current received video speed */
-	if ((rtp->re_video_bytes == 0) && (rtp->re_video_pos == 0)) 
-	  /* Default value for the 1st element */
-	  rtp->re_video_bytes = video_session->GetOctetsReceived();
-	rtp->re_video_speed [rtp->re_video_pos] = 
-	  (float) (video_session->GetOctetsReceived()
-		   - rtp->re_video_bytes)/ 1024;
-	rtp->re_video_bytes = 
-	  video_session->GetOctetsReceived();
-
-	rtp->re_video_pos++;
-	if (rtp->re_video_pos >= 50) rtp->re_video_pos = 0;
-      }
-
-      minutes = t.GetMinutes () % 60;
-      seconds = t.GetSeconds () % 60;
-	
-      gchar *msg = g_strdup_printf 
-	(_("%.2ld:%.2ld:%.2ld  A:%.2f/%.2f   V:%.2f/%.2f"), 
-	 (long) t.GetHours (), (long) minutes, (long) seconds, 
-	 rtp->tr_audio_speed [rtp->tr_audio_pos - 1], 
-	 rtp->re_audio_speed [rtp->re_audio_pos - 1],
-	 rtp->tr_video_speed [rtp->tr_video_pos - 1], 
-	 rtp->re_video_speed [rtp->re_video_pos - 1]);
-	
-
-      if (audio_session) {
-
-	received_packets = audio_session->GetPacketsReceived ();
-	lost_packets = audio_session->GetPacketsLost ();
-	late_packets = audio_session->GetPacketsTooLate ();
-	
-	jitter_buffer_size = audio_session->GetJitterBufferSize ();
-      }
-	
-      if (video_session) {
-
-	received_packets =
-	  received_packets + video_session->GetPacketsReceived ();
-	lost_packets = lost_packets+video_session->GetPacketsLost ();
-	late_packets = late_packets+video_session->GetPacketsTooLate ();
-      }
-
-      if (received_packets > 0) {
-
-	lost_packets_per = ((float) lost_packets * 100.0
-			    / (float) received_packets);
-	late_packets_per = ((float) late_packets * 100.0
-			    / (float) received_packets);
-      }
-      
-      if (t.GetSeconds () > 5) {
-
-	gnomemeeting_statusbar_push (gw->statusbar, msg);
-	gchar *stats_msg = 
-	  g_strdup_printf (_("Lost packets: %.1f %%\nLate packets: %.1f %%\nRound-trip delay: %d ms\nJitter buffer: %d ms"), lost_packets_per, late_packets_per, (int) connection->GetRoundTripDelay().GetMilliSeconds(), int (jitter_buffer_size / 8));
-	gtk_label_set_text (GTK_LABEL (gw->stats_label), stats_msg);
-	g_free (stats_msg);
-
-	gtk_widget_queue_draw_area (gw->stats_drawing_area, 0, 0, GTK_WIDGET (gw->stats_drawing_area)->allocation.width, GTK_WIDGET (gw->stats_drawing_area)->allocation.height);
-      }
-
-      g_free (msg);
-    }
-    
-    gdk_threads_leave ();
-  }
-    
-  return TRUE;
+  if (endpoint && gconf_get_bool (VIDEO_DISPLAY_KEY "stay_on_top")
+      && endpoint->GetCallingState () == GMH323EndPoint::Connected)
+    gdk_window_set_always_on_top (GDK_WINDOW (w->window), TRUE);
 }
 
 
@@ -323,8 +170,8 @@ stats_drawing_area_exposed (GtkWidget *drawing_area, gpointer data)
   int cpt = 0;
   int pos = 0;
   
-  GmWindow *gw = MyApp->GetMainWindow ();
-  GmRtpData *rtp = MyApp->GetRtpData (); 
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
+  GmRtpData *rtp = GnomeMeeting::Process ()->GetRtpData (); 
   GdkPoint points [50];
 
   int width_step = (int) GTK_WIDGET (drawing_area)->allocation.width / 40;
@@ -387,7 +234,7 @@ stats_drawing_area_exposed (GtkWidget *drawing_area, gpointer data)
 
 
   /* Compute the height_step */
-  if (MyApp->Endpoint ()->GetCallingState () == 2) {
+  if (GnomeMeeting::Process ()->Endpoint ()->GetCallingState () == GMH323EndPoint::Connected) {
 
     for (cpt = 0 ; cpt < 50 ; cpt++) {
     
@@ -528,7 +375,7 @@ dnd_drag_data_received_cb (GtkWidget *widget,
   gchar **data_split = NULL;
 
 
-  gw = MyApp->GetMainWindow ();
+  gw = GnomeMeeting::Process ()->GetMainWindow ();
   
   if (selection_data && selection_data->data) {
 
@@ -536,7 +383,7 @@ dnd_drag_data_received_cb (GtkWidget *widget,
 
     if (data_split && data_split [1]) {
 
-      if (MyApp->Endpoint ()->GetCallingState () == 0) {
+      if (GnomeMeeting::Process ()->Endpoint ()->GetCallingState () == GMH323EndPoint::Standby) {
       
 	/* this function will store a copy of text */
 	gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (gw->combo)->entry),
@@ -570,7 +417,7 @@ gnomemeeting_new_event (BonoboListener    *listener,
   char **argv;
   CORBA_sequence_CORBA_string *args;
   
-  GmWindow *gw = MyApp->GetMainWindow ();
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
 
   args = (CORBA_sequence_CORBA_string *) any->_value;
   argc = args->_length;
@@ -592,17 +439,23 @@ gnomemeeting_new_event (BonoboListener    *listener,
   if ((i < argc) && (i + 1 < argc) && (argv [i+1])) {
     
      /* this function will store a copy of text */
-    if (MyApp->Endpoint ()->GetCallingState () == 0) {
+    if (GnomeMeeting::Process ()->Endpoint ()->GetCallingState () == GMH323EndPoint::Standby) {
 
+      gdk_threads_enter ();
       gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (gw->combo)->entry), 
 			  argv [i + 1]);
+      gdk_threads_leave ();
     }
-    
+
+    gdk_threads_enter ();
     connect_cb (NULL, NULL);
+    gdk_threads_leave ();
   }
   else {
 
-    gnomemeeting_warning_dialog (GTK_WINDOW (gm), _("Cannot run GnomeMeeting"), _("GnomeMeeting is already running, if you want it to call a given callto URL, please use \"gnomemeeting -c URL\"."));
+    gdk_threads_enter ();
+    gnomemeeting_warning_dialog (GTK_WINDOW (gm), _("Cannot run GnomeMeeting"), _("GnomeMeeting is already running, if you want it to call a given callto or h323 URL, please use \"gnomemeeting -c URL\"."));
+    gdk_threads_leave ();
   }
 }
 
@@ -718,7 +571,7 @@ main_notebook_page_changed (GtkNotebook *notebook, GtkNotebookPage *page,
 {
   GConfClient *client = gconf_client_get_default ();
 
-  GmWindow *gw = MyApp->GetMainWindow ();
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
 
   gconf_client_set_int (client, VIEW_KEY "control_panel_section",
 			gtk_notebook_get_current_page (GTK_NOTEBOOK (gw->main_notebook)), 0);
@@ -729,53 +582,25 @@ main_notebook_page_changed (GtkNotebook *notebook, GtkNotebookPage *page,
  * DESCRIPTION  :  This callback is called when the user changes the
  *                 audio settings sliders in the main notebook.
  * BEHAVIOR     :  Update the volume of the choosen mixers or of the lid.
- * PRE          :  data = AUDIO_VOLUME or AUDIO_MIC
+ * PRE          :  /
  **/
 void 
 audio_volume_changed (GtkAdjustment *adjustment, gpointer data)
 {
-  int vol_play = 0, vol_rec = 0;
-  gchar *mixer = NULL;
-#ifdef HAS_IXJ
-  GMH323EndPoint *endpoint = NULL;
-  GMLid *lid = NULL;
+  GmWindow *gw = NULL;
+  GMH323EndPoint *ep = NULL;
 
-  endpoint = MyApp->Endpoint ();
-  if (endpoint)
-    lid = endpoint->GetLid ();
-#endif
-  
-  GConfClient *client = gconf_client_get_default ();
-  GmWindow *gw = MyApp->GetMainWindow ();
+  unsigned int play_vol =  0, rec_vol = 0;
 
-  vol_play =  (int) (GTK_ADJUSTMENT (gw->adj_play)->value);
-  vol_rec =  (int) (GTK_ADJUSTMENT (gw->adj_rec)->value);
+  gw = GnomeMeeting::Process ()->GetMainWindow ();
+  ep = GnomeMeeting::Process ()->Endpoint ();
 
-#ifdef HAS_IXJ
-  if (lid) {
+  play_vol = (unsigned int) (GTK_ADJUSTMENT (gw->adj_play)->value);
+  rec_vol = (unsigned int) (GTK_ADJUSTMENT (gw->adj_rec)->value);
 
-    lid->SetVolume (vol_play, vol_rec);
-    lid->Unlock ();
-  }
-  else {
-#endif
-    mixer =
-      gconf_client_get_string (client, DEVICES_KEY "audio_player_mixer",
-			       NULL);
-
-    gnomemeeting_set_mixer_volume (mixer, SOURCE_AUDIO,
-				   ((vol_play<<8)|vol_play));
-    g_free (mixer);
-
-    mixer =
-      gconf_client_get_string (client, DEVICES_KEY "audio_recorder_mixer",
-			       NULL);
-  
-    gnomemeeting_set_mixer_volume (mixer, SOURCE_MIC, ((vol_rec<<8)|vol_rec));
-    g_free (mixer);
-#ifdef HAS_IXJ
-  }
-#endif
+  gdk_threads_leave ();
+  ep->SetDeviceVolume (play_vol, rec_vol);  
+  gdk_threads_enter ();
 }
 
 
@@ -796,12 +621,22 @@ brightness_changed (GtkAdjustment *adjustment, gpointer data)
 
   brightness =  (int) (GTK_ADJUSTMENT (gw->adj_brightness)->value);
 
-  ep = MyApp->Endpoint ();
+
+  /* Notice about mutexes:
+     The GDK lock is taken in the callback. We need to release it, because
+     if CreateVideoGrabber is called in another thread, it will only
+     release its internal mutex (also used by GetVideoGrabber) after it 
+     returns, but it will return only if it is opened, and it can't open 
+     if the GDK lock is held as it will wait on the GDK lock before 
+     updating the GUI */
+  gdk_threads_leave ();
+  ep = GnomeMeeting::Process ()->Endpoint ();
   if (ep && (video_grabber = ep->GetVideoGrabber ())) {
     
     video_grabber->SetBrightness (brightness << 8);
     video_grabber->Unlock ();
   }
+  gdk_threads_enter ();
 }
 
 
@@ -822,12 +657,14 @@ whiteness_changed (GtkAdjustment *adjustment, gpointer data)
 
   whiteness =  (int) (GTK_ADJUSTMENT (gw->adj_whiteness)->value);
 
-  ep = MyApp->Endpoint ();
+  gdk_threads_leave ();
+  ep = GnomeMeeting::Process ()->Endpoint ();
   if (ep && (video_grabber = ep->GetVideoGrabber ())) {
     
     video_grabber->SetWhiteness (whiteness << 8);
     video_grabber->Unlock ();
   }
+  gdk_threads_enter ();
 }
 
 
@@ -848,12 +685,14 @@ colour_changed (GtkAdjustment *adjustment, gpointer data)
 
   colour =  (int) (GTK_ADJUSTMENT (gw->adj_colour)->value);
 
-  ep = MyApp->Endpoint ();
+  gdk_threads_leave ();
+  ep = GnomeMeeting::Process ()->Endpoint ();
   if (ep && (video_grabber = ep->GetVideoGrabber ())) {
     
     video_grabber->SetColour (colour << 8);
     video_grabber->Unlock ();
   }
+  gdk_threads_enter ();
 }
 
 
@@ -874,12 +713,14 @@ contrast_changed (GtkAdjustment *adjustment, gpointer data)
 
   contrast =  (int) (GTK_ADJUSTMENT (gw->adj_contrast)->value);
 
-  ep = MyApp->Endpoint ();
+  gdk_threads_leave ();
+  ep = GnomeMeeting::Process ()->Endpoint ();
   if (ep && (video_grabber = ep->GetVideoGrabber ())) {
     
     video_grabber->SetContrast (contrast << 8);
     video_grabber->Unlock ();
   }
+  gdk_threads_enter ();
 }
 
 
@@ -895,10 +736,15 @@ static void
 dialpad_button_clicked (GtkButton *button, gpointer data)
 { 
   const char *button_text = NULL;
-
+  PString dtmf;
+  
   button_text = gtk_button_get_label (GTK_BUTTON (button));
-  if (button_text)
-    gnomemeeting_dialpad_event (button_text);
+  dtmf = button_text;
+
+  dtmf.Replace ("_", "");
+  
+  if (!dtmf.IsEmpty ())
+    gnomemeeting_dialpad_event (dtmf);
 }
 
 
@@ -913,7 +759,7 @@ gm_quit_callback (GtkWidget *widget, GdkEvent *event,
 			      gpointer data)
 {
   gboolean b;
-  GmWindow *gw = MyApp->GetMainWindow ();
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
 
   b = gnomemeeting_tray_is_visible (gw->docklet);
 
@@ -927,41 +773,18 @@ gm_quit_callback (GtkWidget *widget, GdkEvent *event,
 
 
 /* The functions */
-static int
-gnomemeeting_main_window_appbar_update (gpointer data) 
-{
-  GtkWidget *progress = (GtkWidget *) data;
-  
-  gdk_threads_enter ();
-
-  if (GTK_WIDGET_VISIBLE (GTK_WIDGET (progress)))
-    gtk_progress_bar_pulse (GTK_PROGRESS_BAR (progress));
-
-  gdk_threads_leave ();
-
-  return 1;
-}
-
-
 void
 gnomemeeting_main_window_enable_statusbar_progress (gboolean i)
 {
-  GmWindow *gw = MyApp->GetMainWindow ();
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
   
   if (i) {
     
-    gw->progress_timeout =
-      gtk_timeout_add (2,
-		       gnomemeeting_main_window_appbar_update, 
-		       gw->progressbar);
 
     gtk_widget_show (gw->progressbar);
   }
   else {
 
-    if (gw->progress_timeout != 0)
-      gtk_timeout_remove (gw->progress_timeout);
-    gw->progress_timeout = 0;
 
     gtk_widget_hide (gw->progressbar);
   }    
@@ -980,8 +803,8 @@ void gnomemeeting_dialpad_event (const char *key)
 
   GmWindow *gw = NULL;
 
-  gw = MyApp->GetMainWindow ();
-  endpoint = MyApp->Endpoint ();
+  gw = GnomeMeeting::Process ()->GetMainWindow ();
+  endpoint = GnomeMeeting::Process ()->Endpoint ();
 
   button_text = (gchar *) key;
   url = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (gw->combo)->entry)); 
@@ -991,28 +814,35 @@ void gnomemeeting_dialpad_event (const char *key)
     dtmf = PString (button_text);
     
     /* Replace the * by a . */
-    if (endpoint && endpoint->GetCallingState () == 0
-	&& !strcmp (button_text, "*")) 
+    if (!strcmp (button_text, "*")) 
       button_text = g_strdup (".");
 
-    new_url = url + PString (button_text);
+    if (endpoint && endpoint->GetCallingState () == GMH323EndPoint::Standby) {
 
-    if (!strcmp (".", button_text)) 
-      g_free (button_text);
+      new_url = url + PString (button_text);
 
-    gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (gw->combo)->entry), new_url);
-  
+      if (!strcmp (".", button_text)) 
+	g_free (button_text);
+
+      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (gw->combo)->entry), new_url);
+    }
+
+    
     /* Now we send the pressed key as UserInput */
     if (endpoint) {
         
-      if (endpoint->GetCallingState () == 2) {
+      if (endpoint->GetCallingState () == GMH323EndPoint::Connected) {
             
-	H323Connection *connection = endpoint->GetCurrentConnection ();
+	gdk_threads_leave ();
+	H323Connection *connection = 
+	  endpoint->FindConnectionWithLock (endpoint->GetCurrentCallToken ());
             
 	if (connection) {
 
 	  connection->SendUserInput (dtmf);
+	  connection->Unlock ();
 	}
+	gdk_threads_enter ();
       }
 
 #ifdef HAS_IXJ
@@ -1031,6 +861,73 @@ void gnomemeeting_dialpad_event (const char *key)
 }
 
 
+void
+gnomemeeting_main_window_update_sensitivity (unsigned calling_state)
+{
+  GmWindow *gw = NULL;
+
+  gw = GnomeMeeting::Process ()->GetMainWindow ();
+  
+  switch (calling_state)
+    {
+    case GMH323EndPoint::Standby:
+
+      gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), TRUE);
+      connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 0);
+      break;
+
+
+    case GMH323EndPoint::Calling:
+
+      gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), FALSE);
+      connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 1);
+      break;
+
+
+    case GMH323EndPoint::Connected:
+
+      gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), FALSE);
+      connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 1);
+      break;
+
+
+    case GMH323EndPoint::Called:
+
+      gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), FALSE);
+      break;
+    }
+}
+
+
+void
+gnomemeeting_main_window_update_sensitivity (BOOL is_video,
+					     BOOL is_receiving,
+					     BOOL is_transmitting)
+{
+  GmWindow *gw = NULL;
+  GtkWidget *button = NULL;
+  GtkWidget *frame = NULL;
+  
+  gw = GnomeMeeting::Process ()->GetMainWindow ();
+  
+  if (is_video) {
+
+    frame = gw->video_settings_frame;
+    button = gw->video_chan_button;
+  }
+  else {
+
+    frame = gw->audio_settings_frame;
+    button = gw->audio_chan_button;
+  }
+  
+  gtk_widget_set_sensitive (GTK_WIDGET (button), is_transmitting);
+  gtk_widget_set_sensitive (GTK_WIDGET (frame), is_transmitting);
+  GTK_TOGGLE_BUTTON (button)->active = is_transmitting;
+  gtk_widget_queue_draw (button);
+}
+
+
 GtkWidget *
 gnomemeeting_main_window_new (GmWindow *gw)
 {
@@ -1048,7 +945,7 @@ gnomemeeting_main_window_new (GmWindow *gw)
   GtkWidget *event_box = NULL;
   GtkWidget *main_toolbar = NULL;
   GtkWidget *left_toolbar = NULL;
-  GtkWidget *menubar = NULL;
+
   int main_notebook_section = 0;
   int x = GM_QCIF_WIDTH;
   int y = GM_QCIF_HEIGHT;
@@ -1062,7 +959,7 @@ gnomemeeting_main_window_new (GmWindow *gw)
   };
 
   client = gconf_client_get_default ();
-  chat = MyApp->GetTextChat ();
+  chat = GnomeMeeting::Process ()->GetTextChat ();
   
   accel = gtk_accel_group_new ();
   gtk_window_add_accel_group (GTK_WINDOW (window), accel);
@@ -1073,15 +970,45 @@ gnomemeeting_main_window_new (GmWindow *gw)
   gtk_widget_show (window_vbox);
 #endif
 
-  menubar = gnomemeeting_init_menu (accel);
+    /* The statusbar and the progressbar */
+  hbox = gtk_hbox_new (0, FALSE);
+#ifdef DISABLE_GNOME
+  gtk_box_pack_start (GTK_BOX (window_vbox), hbox, 
+		      FALSE, FALSE, 0);
+#else
+  gnome_app_add_docked (GNOME_APP (window), hbox, "statusbar",
+  			BONOBO_DOCK_ITEM_BEH_EXCLUSIVE,
+  			BONOBO_DOCK_BOTTOM, 3, 0, 0);
+#endif
+  gtk_widget_show (hbox);
+
+  
+  gw->progressbar = gtk_progress_bar_new ();
+  gtk_widget_set_size_request (GTK_WIDGET (gw->progressbar), 25, -1);
+  gtk_box_pack_start (GTK_BOX (hbox), gw->progressbar, 
+		      FALSE, FALSE, 0);
+
+  gw->statusbar = gtk_statusbar_new ();
+  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (gw->statusbar), FALSE);
+  gtk_box_pack_start (GTK_BOX (hbox), gw->statusbar, 
+		      TRUE, TRUE, 0);
+
+  if (gconf_client_get_bool (client, 
+			     VIEW_KEY "show_status_bar", 0))
+    gtk_widget_show (GTK_WIDGET (gw->statusbar));
+  else
+    gtk_widget_hide (GTK_WIDGET (gw->statusbar));
+
+  
+  gw->main_menu = gnomemeeting_init_menu (accel);
 #ifndef DISABLE_GNOME
   gnome_app_add_docked (GNOME_APP (window), 
-			menubar,
+			gw->main_menu,
 			"menubar",
 			BONOBO_DOCK_ITEM_BEH_EXCLUSIVE,
   			BONOBO_DOCK_TOP, 0, 0, 0);
 #else
-  gtk_box_pack_start (GTK_BOX (window_vbox), menubar,
+  gtk_box_pack_start (GTK_BOX (window_vbox), gw->main_menu,
 		      FALSE, FALSE, 0);
 #endif
 
@@ -1112,8 +1039,8 @@ gnomemeeting_main_window_new (GmWindow *gw)
 
   gtk_widget_show (main_toolbar);
   gtk_widget_show (left_toolbar);
-  gtk_widget_show (menubar);
 
+  
   /* Create a table in the main window to attach things like buttons */
   table = gtk_table_new (3, 4, FALSE);
 #ifdef DISABLE_GNOME
@@ -1131,7 +1058,7 @@ gnomemeeting_main_window_new (GmWindow *gw)
   gtk_notebook_set_scrollable (GTK_NOTEBOOK (gw->main_notebook), TRUE);
 
   gnomemeeting_init_main_window_stats ();
-  gnomemeeting_init_main_window_dialpad ();
+  gnomemeeting_init_main_window_dialpad (accel);
   gnomemeeting_init_main_window_audio_settings ();
   gnomemeeting_init_main_window_video_settings ();
 
@@ -1163,7 +1090,7 @@ gnomemeeting_main_window_new (GmWindow *gw)
   gtk_frame_set_shadow_type (GTK_FRAME (gw->video_frame), GTK_SHADOW_IN);
   
   event_box = gtk_event_box_new ();
-  gnomemeeting_popup_menu_init (event_box, accel);
+  gw->video_popup_menu = gnomemeeting_video_popup_init_menu (event_box, accel);
 
   vbox = gtk_vbox_new (FALSE, 0);
 
@@ -1190,19 +1117,24 @@ gnomemeeting_main_window_new (GmWindow *gw)
 
   
   /* The 2 video window popups */
-  x = gconf_client_get_int (client, "/apps/gnomemeeting/video_display/local_video_width", NULL);
-  y = gconf_client_get_int (client, "/apps/gnomemeeting/video_display/local_video_height", NULL);
+  x = gconf_client_get_int (client, VIDEO_DISPLAY_KEY "local_video_width", NULL);
+  y = gconf_client_get_int (client, VIDEO_DISPLAY_KEY "local_video_height", NULL);
   gw->local_video_window =
     gnomemeeting_video_window_new (_("Local Video"), gw->local_video_image,
 				   x, y);
 
-  x = gconf_client_get_int (client, "/apps/gnomemeeting/video_display/remote_video_width", NULL);
-  y = gconf_client_get_int (client, "/apps/gnomemeeting/video_display/remote_video_height", NULL);
+  x = gconf_client_get_int (client, VIDEO_DISPLAY_KEY "remote_video_width", NULL);
+  y = gconf_client_get_int (client, VIDEO_DISPLAY_KEY "remote_video_height", NULL);
   gw->remote_video_window =
     gnomemeeting_video_window_new (_("Remote Video"), gw->remote_video_image,
 				   x, y);
   gnomemeeting_init_main_window_logo (gw->main_video_image);
 
+  g_signal_connect (G_OBJECT (gw->local_video_window), "show", 
+		    GTK_SIGNAL_FUNC (video_window_shown_cb), NULL);
+  g_signal_connect (G_OBJECT (gw->remote_video_window), "show", 
+		    GTK_SIGNAL_FUNC (video_window_shown_cb), NULL);
+
 
   /* The remote name */
   frame = gtk_frame_new (NULL);
@@ -1225,51 +1157,14 @@ gnomemeeting_main_window_new (GmWindow *gw)
  		    (GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
  		    6, 6);
   if (gconf_client_get_bool 
-      (client, "/apps/gnomemeeting/view/show_chat_window", 0))
+      (client, VIEW_KEY "show_chat_window", 0))
     gtk_widget_show_all (GTK_WIDGET (gw->chat_window));
-
-
-  /* The statusbar and the progressbar */
-  hbox = gtk_hbox_new (0, FALSE);
-#ifdef DISABLE_GNOME
-  gtk_box_pack_start (GTK_BOX (window_vbox), hbox, 
-		      FALSE, FALSE, 0);
-#else
-  gnome_app_add_docked (GNOME_APP (window), hbox, "statusbar",
-  			BONOBO_DOCK_ITEM_BEH_EXCLUSIVE,
-  			BONOBO_DOCK_BOTTOM, 3, 0, 0);
-#endif
-  gtk_widget_show (hbox);
-  
-  gw->progressbar = gtk_progress_bar_new ();
-  gtk_widget_set_size_request (GTK_WIDGET (gw->progressbar), 25, -1);
-  gtk_box_pack_start (GTK_BOX (hbox), gw->progressbar, 
-		      FALSE, FALSE, 0);
-
-  gw->statusbar = gtk_statusbar_new ();
-  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (gw->statusbar), FALSE);
-  gtk_box_pack_start (GTK_BOX (hbox), gw->statusbar, 
-		      TRUE, TRUE, 0);
-
-  if (gconf_client_get_bool (client, 
-			     VIEW_KEY "show_status_bar", 0))
-    gtk_widget_show (GTK_WIDGET (gw->statusbar));
-  else
-    gtk_widget_hide (GTK_WIDGET (gw->statusbar));
-
   
   gtk_widget_set_size_request (GTK_WIDGET (gw->main_notebook),
 			       GM_QCIF_WIDTH + GM_FRAME_SIZE, -1);
   gtk_widget_set_size_request (GTK_WIDGET (window), -1, -1);
 
   
-  /* Add the popup menu and change all menus sensitivity */
-  gnomemeeting_video_submenu_set_sensitive (FALSE, LOCAL_VIDEO);
-  gnomemeeting_video_submenu_set_sensitive (FALSE, REMOTE_VIDEO);
-  gnomemeeting_zoom_submenu_set_sensitive (FALSE);
-  gnomemeeting_fullscreen_option_set_sensitive (FALSE);
-
-
   /* Add the window icon and title */
   gtk_window_set_title (GTK_WINDOW (window), _("GnomeMeeting"));
   pixbuf = 
@@ -1296,9 +1191,6 @@ gnomemeeting_main_window_new (GmWindow *gw)
   g_signal_connect (G_OBJECT (gm), "delete_event",
 		    G_CALLBACK (gm_quit_callback), (gpointer) gw);
 
-  GmRtpData *rtp = MyApp->GetRtpData (); /* Fix ME: shouldn't always run*/
-  gtk_timeout_add (1000, (GtkFunction) AppbarUpdate, rtp);
-
   
   return window;
 }
@@ -1315,7 +1207,7 @@ void gnomemeeting_init_main_window_stats ()
   GtkWidget *label = NULL;
   GtkWidget *vbox = NULL;
 
-  GmWindow *gw = MyApp->GetMainWindow ();
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
 
   /* The first frame with statistics display */
   frame2 = gtk_frame_new (NULL);
@@ -1377,7 +1269,7 @@ void gnomemeeting_init_main_window_stats ()
  * BEHAVIOR     :  Builds the dialpad part of the main window.
  * PRE          :  /
  **/
-void gnomemeeting_init_main_window_dialpad ()
+void gnomemeeting_init_main_window_dialpad (GtkAccelGroup *accel)
 {
   GtkWidget *label = NULL;
   GtkWidget *table = NULL;
@@ -1388,7 +1280,7 @@ void gnomemeeting_init_main_window_dialpad ()
   int j = 0;
   int j2 = 0;
 
-  GmWindow *gw = MyApp->GetMainWindow ();
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
 
   table = gtk_table_new (4, 3, TRUE);
   gtk_container_set_border_width (GTK_CONTAINER (table), 3);
@@ -1398,28 +1290,28 @@ void gnomemeeting_init_main_window_dialpad ()
       if (j == 0) {
 
 	if (i == 1)
-	  button_text = g_strdup ("*");
+	  button_text = g_strdup ("_*");
 	else if (i == 2)
-	  button_text = g_strdup ("0");
+	  button_text = g_strdup ("_0");
 	else if (i == 3)
-	  button_text = g_strdup ("#");
+	  button_text = g_strdup ("_#");
 	
 	j2 = 3;
       }
       else {
 
 	j2 = j - 1;
-	button_text = g_strdup_printf ("%d", (j - 1) * 3 + i);
+	button_text = g_strdup_printf ("_%d", (j - 1) * 3 + i);
       }
 
-      button = gtk_button_new_with_label (button_text);
+      button = gtk_button_new_with_mnemonic (button_text);
 
       gtk_table_attach (GTK_TABLE (table), GTK_WIDGET (button), 
 			i - 1, i, j2, j2+1,
 			(GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
 			(GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
 			1, 1);
-
+      
       g_signal_connect (G_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (dialpad_button_clicked), NULL);
 
@@ -1451,7 +1343,7 @@ void gnomemeeting_init_main_window_video_settings ()
 
   int brightness = 0, colour = 0, contrast = 0, whiteness = 0;
   
-  GmWindow *gw = MyApp->GetMainWindow ();
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
 
   
   /* Webcam Control Frame, we need it to disable controls */		
@@ -1574,16 +1466,22 @@ void gnomemeeting_init_main_window_audio_settings ()
   GtkWidget *hbox = NULL;
   GtkWidget *vbox = NULL;
 
-  char *mixer = NULL;
-  int vol = 0;
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
 
-  GConfClient *client = gconf_client_get_default ();
-  GmWindow *gw = MyApp->GetMainWindow ();
 
-  
+  /* Webcam Control Frame, we need it to disable controls */		
+  gw->audio_settings_frame = gtk_frame_new (NULL);
+  gtk_frame_set_shadow_type (GTK_FRAME (gw->audio_settings_frame), 
+			     GTK_SHADOW_NONE);
+  gtk_container_set_border_width (GTK_CONTAINER (gw->audio_settings_frame), 0);
+
+
+  /* The vbox */
   vbox = gtk_vbox_new (0, FALSE);
-
+  gtk_container_add (GTK_CONTAINER (gw->audio_settings_frame), vbox);
+  gtk_widget_set_sensitive (GTK_WIDGET (gw->audio_settings_frame), FALSE);
   
+
   /* Audio volume */
   hbox = gtk_hbox_new (0, FALSE);
   gtk_box_pack_start (GTK_BOX (hbox),
@@ -1591,12 +1489,7 @@ void gnomemeeting_init_main_window_audio_settings ()
 						GTK_ICON_SIZE_SMALL_TOOLBAR),
 		      FALSE, FALSE, 0);
   
-  mixer =
-    gconf_client_get_string (client, DEVICES_KEY "audio_player_mixer", NULL);
-  vol = gnomemeeting_get_mixer_volume (mixer, SOURCE_AUDIO);
-  g_free (mixer);
-  
-  gw->adj_play = gtk_adjustment_new ((vol&255), 0.0, 100.0, 1.0, 5.0, 1.0);
+  gw->adj_play = gtk_adjustment_new (0, 0.0, 100.0, 1.0, 5.0, 1.0);
   hscale_play = gtk_hscale_new (GTK_ADJUSTMENT (gw->adj_play));
   gtk_range_set_update_policy (GTK_RANGE (hscale_play),
 			       GTK_UPDATE_DELAYED);
@@ -1613,12 +1506,7 @@ void gnomemeeting_init_main_window_audio_settings ()
 						GTK_ICON_SIZE_SMALL_TOOLBAR),
 		      FALSE, FALSE, 0);
 
-  mixer =
-    gconf_client_get_string (client, DEVICES_KEY "audio_recorder_mixer", NULL);
-  vol = gnomemeeting_get_mixer_volume (mixer, SOURCE_MIC);
-  g_free (mixer);
-  
-  gw->adj_rec = gtk_adjustment_new ((vol&255), 0.0, 100.0, 1.0, 5.0, 1.0);
+  gw->adj_rec = gtk_adjustment_new (0, 0.0, 100.0, 1.0, 5.0, 1.0);
   hscale_rec = gtk_hscale_new (GTK_ADJUSTMENT (gw->adj_rec));
   gtk_range_set_update_policy (GTK_RANGE (hscale_rec),
 			       GTK_UPDATE_DELAYED);
@@ -1638,7 +1526,7 @@ void gnomemeeting_init_main_window_audio_settings ()
   label = gtk_label_new (_("Audio"));
 
   gtk_notebook_append_page (GTK_NOTEBOOK (gw->main_notebook),
-			    vbox, label);
+			    gw->audio_settings_frame, label);
 }
 
 
@@ -1649,9 +1537,15 @@ int main (int argc, char ** argv, char ** envp)
   PProcess::PreInitialise (argc, argv, envp);
 
   GtkWidget *dialog = NULL;
+  
   GmWindow *gw = NULL;
+
   gchar *url = NULL;
+  gchar *key_name = NULL;
+  gchar *msg = NULL;
+
   int debug_level = 0;
+
   
 #ifndef WIN32
 #if !defined(HAS_ESD)
@@ -1672,10 +1566,7 @@ int main (int argc, char ** argv, char ** envp)
   gtk_init (NULL, NULL);
 #endif
 
-
-  /* Init libxml, could add an XDAP function for that: FIX ME */
   xmlInitParser ();
-
   
 #ifndef DISABLE_GNOME
   /* Cope with command line options */
@@ -1703,6 +1594,11 @@ int main (int argc, char ** argv, char ** envp)
 #else
   gm = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 #endif
+  g_object_set_data_full (G_OBJECT (gm), "window_name",
+			  g_strdup ("main_window"), g_free);
+  
+  g_signal_connect (G_OBJECT (gm), "show", 
+		    GTK_SIGNAL_FUNC (video_window_shown_cb), NULL);
   
   gdk_threads_enter ();
   gconf_init (argc, argv, 0);
@@ -1718,64 +1614,72 @@ int main (int argc, char ** argv, char ** envp)
   if (gnomemeeting_invoke_factory (argc, argv))
     exit (1);
 #endif
-  
 
+  
+  gnomemeeting_warning_dialog (NULL, "WARNING!", "You are using a CVS version of GnomeMeeting, use it at your own risk!");
+
+  
   /* Upgrade the preferences */
   gnomemeeting_gconf_upgrade ();
 
   
   /* GnomeMeeting main initialisation */
-  MyApp = new GnomeMeeting;
-
+  static GnomeMeeting instance;
 
+  
   /* Detect the devices, exit if it fails */
-  if (!MyApp->DetectDevices ()) {
-    
-    dialog = gnomemeeting_error_dialog (NULL, _("No usable audio devices detected"), _("GnomeMeeting didn't find any usable sound devices. You need to install and setup a soundcard or a Quicknet card in order to be able to use GnomeMeeting. Please check your driver settings and permissions."));
+  if (!GnomeMeeting::Process ()->DetectDevices ()) {
 
+    dialog = gnomemeeting_error_dialog (NULL, _("No usable audio manager detected"), _("GnomeMeeting didn't find any usable sound manager. Make sure that your installation is correct."));
+    
     g_signal_handlers_disconnect_by_func (G_OBJECT (dialog),
 					  (gpointer) gtk_widget_destroy,
 					  G_OBJECT (dialog));
 
     gtk_dialog_run (GTK_DIALOG (dialog));
-    delete (MyApp);
     exit (-1);
   }
 
 
-  /* Build the GUI and init the different components */
-  MyApp->BuildGUI ();
-  MyApp->InitComponents ();
+  /* Init the process and build the GUI */
+  GnomeMeeting::Process ()->BuildGUI ();
+  GnomeMeeting::Process ()->Init ();
 
 
   /* Init the GConf DB, exit if it fails */
   if (!gnomemeeting_init_gconf (gconf_client_get_default ())) {
 
-    dialog = gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Gconf key error"), _("GnomeMeeting got an invalid value for the GConf key \"/apps/gnomemeeting/gconf_test_age\".\n\nIt probably means that your GConf schemas have not been correctly installed or the that permissions are not correct.\n\nPlease check the FAQ (http://www.gnomemeeting.org/faq.php), the throubleshoot section of the GConf site (http://www.gnome.org/projects/gconf/) or the mailing list archives for more information (http://mail.gnome.org) about this problem."));
+    key_name = g_strdup ("\"/apps/gnomemeeting/general/gconf_test_age\"");
+    msg = g_strdup_printf (_("GnomeMeeting got an invalid value for the GConf key %s.\n\nIt probably means that your GConf schemas have not been correctly installed or the that permissions are not correct.\n\nPlease check the FAQ (http://www.gnomemeeting.org/faq.php), the throubleshoot section of the GConf site (http://www.gnome.org/projects/gconf/) or the mailing list archives for more information (http://mail.gnome.org) about this problem."), key_name);
+    
+    dialog = gnomemeeting_error_dialog (GTK_WINDOW (gm),
+					_("Gconf key error"), msg);
 
     g_signal_handlers_disconnect_by_func (G_OBJECT (dialog),
 					  (gpointer) gtk_widget_destroy,
 					  G_OBJECT (dialog));
 
+
+    g_free (msg);
+    g_free (key_name);
     
     gtk_dialog_run (GTK_DIALOG (dialog));
-    delete (MyApp);
     exit (-1);
   }
 
   
   /* Debug */
   if (debug_level != 0)
-    PTrace::Initialise (PMAX (PMIN (4, debug_level), 0),
-			"gnomemeeting-debug.log",
+    PTrace::Initialise (PMAX (PMIN (4, debug_level), 0));
+  /*			"gnomemeeting-debug.log",
 			PTrace::Timestamp | PTrace::Thread
 			| PTrace::Blocks | PTrace::DateAndTime);
-
+  */
 
   /* Call the given host if needed */
   if (url) {
 
-    gw = MyApp->GetMainWindow ();
+    gw = GnomeMeeting::Process ()->GetMainWindow ();
     gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (gw->combo)->entry), url);
     connect_cb (NULL, NULL);
   }
@@ -1783,12 +1687,11 @@ int main (int argc, char ** argv, char ** envp)
   //  gtk_timeout_add (15000, (GtkFunction) StressTest, 
   //		   NULL);
   
-
   /* The GTK loop */
   gtk_main ();
   gdk_threads_leave ();
 
-  delete (MyApp);
+  //  delete (GnomeMeeting::Process ());
   
 #ifdef DISABLE_GCONF
   gconf_save_content_to_file ();
