@@ -74,19 +74,34 @@ void gnomemeeting_sound_daemons_suspend ();
 void gnomemeeting_sound_daemons_resume ();
 
 
-/* DESCRIPTION  :  This callback is called by a timer function.
- * BEHAVIOR     :  Plays the sound choosen in the gnome control center.
- * PRE          :  The pointer to the docklet must be valid.
+/* DESCRIPTION  :  /
+ * BEHAVIOR     :  Selects the Mic as recording input. OSS only.
+ * PRE          :  /
  */
-int gnomemeeting_sound_play_ringtone (GtkWidget *widget);
-
 void gnomemeeting_mixers_mic_select (void);
-PStringArray gnomemeeting_get_mixers (void);
-PStringArray gnomemeeting_get_audio_player_devices (void);
-PStringArray gnomemeeting_get_audio_recorder_devices (void);
-int gnomemeeting_get_mixer_volume (char *mixer, int source);
-void gnomemeeting_set_mixer_volume (char *mixer, int source, int vol);
      
+
+/* GnomeMeeting Sound Event class */
+class GMSoundEvent
+{
+
+ public:
+
+  /* DESCRIPTION  :  The constructor.
+   * BEHAVIOR     :  Builds the PSound corresponding to the event name
+   *                 and plays it if it is valid.
+   * PRE          :  The sound event name.
+   */
+  GMSoundEvent (PString);
+
+ private:
+  void Main ();
+
+  PString event;
+};
+
+
+/* Audio Tester class */
 class GMAudioTester : public PThread
 {
   PCLASSINFO(GMAudioTester, PThread);
@@ -95,10 +110,12 @@ class GMAudioTester : public PThread
 public:
 
   /* DESCRIPTION  :  The constructor.
-   * BEHAVIOR     :  
-   * PRE          :  
+   * BEHAVIOR     :  /
+   * PRE          :  /
    */
-  GMAudioTester (GMH323EndPoint *);
+  GMAudioTester (gchar *,
+		 gchar *,
+		 gchar *);
 
 
   /* DESCRIPTION  :  The destructor.
@@ -110,19 +127,53 @@ public:
 
   void Main ();
 
-
-  void Stop ();
-
-
 protected:
 
   BOOL stop;
+
   PMutex quit_mutex;
+  PSyncPoint thread_sync_point;
+
   PSoundChannel *player;
   PSoundChannel *recorder;
 
-  GmWindow *gw;
+  char *buffer_ring;
 
-  GMH323EndPoint *ep;
+  PString audio_manager;
+  PString audio_player;
+  PString audio_recorder;
+  
+  PMutex buffer_ring_access_mutex;
+
+  GmWindow *gw;
+  
+  GtkWidget *test_label;
+  GtkWidget *test_dialog;
+
+  friend class GMAudioRP;
 };
 #endif
+
+
+class GMAudioRP : public PThread
+{
+  PCLASSINFO(GMAudioRP, PThread);
+
+ public:
+
+  GMAudioRP (GMAudioTester *, PString, PString, BOOL);
+  ~GMAudioRP ();
+
+  void Main ();
+
+ private:
+
+  BOOL is_encoding;
+  GMAudioTester *tester;
+  PString driver_name;
+  PString device_name;
+  PMutex quit_mutex;
+  PSyncPoint thread_sync_point;
+  
+  BOOL stop;
+};
