@@ -45,13 +45,12 @@
 #include "callbacks.h"
 #include "config.h"
 #include "misc.h" 
+
 #include "stock-icons.h"
 #include "history-combo.h"
-
+#include "gconf_widgets_extensions.h"
 
 /* Declarations */
-
-extern GnomeMeeting *MyApp;
 extern GtkWidget *gm;
 
 
@@ -65,7 +64,7 @@ static void toolbar_cp_button_changed    (GtkWidget *, gpointer);
 /* Static functions */
 static void url_combo_changed (GtkEditable  *e, gpointer data)
 {
-  GmWindow *gw = MyApp->GetMainWindow ();
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
 
   gchar *tip_text = (gchar *)
     gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (data)->entry));
@@ -83,12 +82,12 @@ static void url_combo_changed (GtkEditable  *e, gpointer data)
 static void connect_button_clicked (GtkToggleButton *w, gpointer data)
 {
   if (gtk_toggle_button_get_active (w))
-    if (MyApp->Endpoint ()->GetCallingState () == 0) 
-      MyApp->Connect ();
+    if (GnomeMeeting::Process ()->Endpoint ()->GetCallingState () == GMH323EndPoint::Standby) 
+      GnomeMeeting::Process ()->Connect ();
     else 
       connect_button_update_pixmap (w, 1);
   else
-    MyApp->Disconnect ();
+    GnomeMeeting::Process ()->Disconnect ();
 }
 
 
@@ -139,7 +138,7 @@ gnomemeeting_init_main_toolbar ()
   GtkWidget *hbox = NULL;
   GtkWidget *image = NULL;
 
-  GmWindow *gw = MyApp->GetMainWindow ();
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
   
 
   /* The main horizontal toolbar */
@@ -208,9 +207,8 @@ GtkWidget *gnomemeeting_init_left_toolbar (void)
 
   PString dev;
   
-  GmWindow *gw = MyApp->GetMainWindow ();
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
 
-  GConfClient *client = gconf_client_get_default ();
 
   left_toolbar = gtk_toolbar_new ();
   gtk_toolbar_set_orientation (GTK_TOOLBAR (left_toolbar), 
@@ -248,7 +246,7 @@ GtkWidget *gnomemeeting_init_left_toolbar (void)
 			   _("Open address book"),
 			   NULL,
 			   image,
-			   GTK_SIGNAL_FUNC (gnomemeeting_component_view),
+			   GTK_SIGNAL_FUNC (show_window_cb),
 			   (gpointer) gw->ldap_window); 
 
   gtk_toolbar_set_style (GTK_TOOLBAR (left_toolbar), GTK_TOOLBAR_ICONS);
@@ -261,18 +259,12 @@ GtkWidget *gnomemeeting_init_left_toolbar (void)
                                     GTK_ICON_SIZE_MENU);
 
   gtk_container_add (GTK_CONTAINER (gw->preview_button), GTK_WIDGET (image));
+  GTK_TOGGLE_BUTTON (gw->preview_button)->active =
+    gconf_get_bool (VIDEO_DEVICES_KEY "enable_preview");
 
-  /* We set the key as data to be able to get the data in order to block       
-     the signal in the gconf notifier */                             
-  g_object_set_data (G_OBJECT (gw->preview_button), "gconf_key", 
-		     (void *) DEVICES_KEY "video_preview");
-
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gw->preview_button), 
-				gconf_client_get_bool (client, DEVICES_KEY "video_preview", NULL));
-
-  g_signal_connect (G_OBJECT (gw->preview_button), "clicked",
-		    G_CALLBACK (toggle_changed), 
-		    (gpointer) DEVICES_KEY "video_preview");
+  g_signal_connect (G_OBJECT (gw->preview_button), "toggled",
+		    G_CALLBACK (toggle_changed),
+		    (gpointer) VIDEO_DEVICES_KEY "enable_preview");
 
   gtk_tooltips_set_tip (gw->tips, gw->preview_button,
 			_("Display images from your camera device."),
@@ -335,7 +327,7 @@ void connect_button_update_pixmap (GtkToggleButton *button, int pressed)
 {
   GtkWidget *image = NULL;
   
-  GmWindow *gw = MyApp->GetMainWindow ();
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
   
   image = (GtkWidget *) g_object_get_data (G_OBJECT (button), "image");
   
