@@ -48,7 +48,6 @@
 
 
 extern GtkWidget *gm;
-extern GnomeMeeting *MyApp;
 
 
 static void dnd_drag_data_get_cb (GtkWidget *,
@@ -115,6 +114,7 @@ gnomemeeting_calls_history_window_add_call (int i,
 					    const char *remote_user,
 					    const char *ip,
 					    const char *duration,
+					    const char *reason,
 					    const char *software)
 {
   GtkListStore *list_store = NULL;
@@ -126,7 +126,7 @@ gnomemeeting_calls_history_window_add_call (int i,
   
   GmCallsHistoryWindow *chw = NULL;
 
-  chw = MyApp->GetCallsHistoryWindow ();
+  chw = GnomeMeeting::Process ()->GetCallsHistoryWindow ();
   
   switch (i) {
 
@@ -159,7 +159,8 @@ gnomemeeting_calls_history_window_add_call (int i,
 			1, remote_user ? remote_user : "",
 			2, ip ? ip : "",
 			3, duration ? duration : "",
-			4, software ? software : "",
+			4, reason ? reason : "",
+			5, software ? software : "",
 			-1);
 
     g_free (utf8_time);
@@ -183,7 +184,7 @@ gnomemeeting_calls_history_window_new (GmCallsHistoryWindow *chw)
   GtkListStore *list_store [3];
   
   gchar *label_text [3] =
-    {N_("Received calls"), N_("Given calls"), N_("Unanswered calls")};
+    {N_("Received Calls"), N_("Placed Calls"), N_("Unanswered Calls")};
   label_text [0] = gettext (label_text [0]);
   label_text [1] = gettext (label_text [1]);
   label_text [2] = gettext (label_text [2]);
@@ -196,10 +197,11 @@ gnomemeeting_calls_history_window_new (GmCallsHistoryWindow *chw)
   
   window = gtk_dialog_new ();
   gtk_dialog_add_button (GTK_DIALOG (window), GTK_STOCK_CLOSE, 0);
-
+  g_object_set_data_full (G_OBJECT (window), "window_name",
+			  g_strdup ("calls_history_window"), g_free);
+  
   gtk_window_set_title (GTK_WINDOW (window), _("Calls History"));
   gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
-  gtk_window_set_default_size (GTK_WINDOW (window), 330, 225);
   icon = gtk_widget_render_icon (GTK_WIDGET (window),
 				 GM_STOCK_CALLS_HISTORY,
 				 GTK_ICON_SIZE_MENU, NULL);
@@ -220,8 +222,14 @@ gnomemeeting_calls_history_window_new (GmCallsHistoryWindow *chw)
 				    GTK_POLICY_AUTOMATIC);
 
     list_store [i] = 
-      gtk_list_store_new (6, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-			  G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+      gtk_list_store_new (6,
+			  G_TYPE_STRING,
+			  G_TYPE_STRING,
+			  G_TYPE_STRING,
+			  G_TYPE_STRING,
+			  G_TYPE_STRING,
+			  G_TYPE_STRING,
+			  G_TYPE_STRING);
     
     tree_view = 
       gtk_tree_view_new_with_model (GTK_TREE_MODEL (list_store [i]));
@@ -236,7 +244,7 @@ gnomemeeting_calls_history_window_new (GmCallsHistoryWindow *chw)
     gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
     
     renderer = gtk_cell_renderer_text_new ();
-    column = gtk_tree_view_column_new_with_attributes (_("Remote user"),
+    column = gtk_tree_view_column_new_with_attributes (_("Remote User"),
 						       renderer,
 						       "text", 
 						       1,
@@ -251,11 +259,10 @@ gnomemeeting_calls_history_window_new (GmCallsHistoryWindow *chw)
 						       2,
 						       NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
-    g_object_set (G_OBJECT (renderer), "foreground", "blue",
-		  "underline", TRUE, NULL);
+    gtk_tree_view_column_set_visible (column, false);
 
     renderer = gtk_cell_renderer_text_new ();
-    column = gtk_tree_view_column_new_with_attributes (_("Call duration"),
+    column = gtk_tree_view_column_new_with_attributes (_("Call Duration"),
 						       renderer,
 						       "text", 
 						       3,
@@ -264,12 +271,20 @@ gnomemeeting_calls_history_window_new (GmCallsHistoryWindow *chw)
     if (i == 2)
       gtk_tree_view_column_set_visible (column, false);
 
-
+    renderer = gtk_cell_renderer_text_new ();
+    column = gtk_tree_view_column_new_with_attributes (_("Call End Reason"),
+						       renderer,
+						       "text", 
+						       4,
+						       NULL);
+    gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
+    g_object_set (G_OBJECT (renderer), "style", PANGO_STYLE_ITALIC, NULL);
+    
     renderer = gtk_cell_renderer_text_new ();
     column = gtk_tree_view_column_new_with_attributes (_("Software"),
 						       renderer,
 						       "text", 
-						       4,
+						       5,
 						       NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
     g_object_set (G_OBJECT (renderer), "style", PANGO_STYLE_ITALIC, NULL);
@@ -299,14 +314,18 @@ gnomemeeting_calls_history_window_new (GmCallsHistoryWindow *chw)
   chw->given_calls_list_store = list_store [1];
   chw->missed_calls_list_store = list_store [2];
 
-  g_signal_connect (G_OBJECT (window), "delete_event",
-		    G_CALLBACK (gtk_widget_hide_on_delete), NULL);
-
   g_signal_connect_swapped (GTK_OBJECT (window), 
 			    "response", 
-			    G_CALLBACK (gtk_widget_hide_all),
+			    G_CALLBACK (gnomemeeting_window_hide),
 			    (gpointer) window);
 
+  g_signal_connect_swapped (GTK_OBJECT (window), 
+			    "delete-event", 
+			    G_CALLBACK (gtk_widget_hide_on_delete),
+			    (gpointer) window);
+
+  gtk_widget_show_all (GTK_WIDGET (GTK_DIALOG (window)->vbox));
+  
   return window;
 }
 
@@ -319,17 +338,18 @@ GtkWidget *gnomemeeting_history_window_new ()
   GtkTextBuffer *buffer = NULL;
   GtkTextIter end;
 
-  GmWindow *gw = MyApp->GetMainWindow ();
+  GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
 
   /* Fix me, create a structure for that so that we don't use
      gw here */
   window = gtk_dialog_new ();
   gtk_dialog_add_button (GTK_DIALOG (window), GTK_STOCK_CLOSE, 0);
-
+  g_object_set_data_full (G_OBJECT (window), "window_name",
+			  g_strdup ("general_history_window"), g_free);
+  
   gtk_window_set_title (GTK_WINDOW (window), _("General History"));
   gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
-  gtk_window_set_default_size (GTK_WINDOW (window), 330, 225);
-
+  
   gw->history_text_view = gtk_text_view_new ();
   gtk_text_view_set_editable (GTK_TEXT_VIEW (gw->history_text_view), 
 			      FALSE);
@@ -357,14 +377,18 @@ GtkWidget *gnomemeeting_history_window_new ()
   gtk_container_add (GTK_CONTAINER (scr), gw->history_text_view);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), scr,
 		      TRUE, TRUE, 0);
- 
-  g_signal_connect (G_OBJECT (window), "delete_event",
-		    G_CALLBACK (gtk_widget_hide_on_delete), NULL);
-
+  
   g_signal_connect_swapped (GTK_OBJECT (window), 
 			    "response", 
-			    G_CALLBACK (gtk_widget_hide_all),
+			    G_CALLBACK (gnomemeeting_window_hide),
 			    (gpointer) window);
 
+  g_signal_connect_swapped (GTK_OBJECT (window), 
+			    "delete-event", 
+			    G_CALLBACK (gtk_widget_hide_on_delete),
+			    (gpointer) window);
+  
+  gtk_widget_show_all (GTK_WIDGET (GTK_DIALOG (window)->vbox));
+  
   return window;
 }
